import { parseTime } from '@/utils/tools'
import {
  queryAttendanceGroupByUserId,
  checkIn,
  checkOut,
  getCurrentDayRecord
} from '@/api/checkin/punch-card'

// 打卡状态常量
export const PUNCH_STATUS = {
  NOT_CHECKED: 0,
  CHECKED_IN: 1,
  CHECKED_OUT: 2,
  LATE_IN: 3,
  EARLY_OUT: 4,
  UPDATE_OUT: 5
}

// 打卡类型常量
export const PUNCH_TYPE = {
  CLOCK_IN: 'clockIn',
  CLOCK_OUT: 'clockOut'
}

/**
 * 考勤服务类
 * 处理所有考勤相关的业务逻辑
 */
export class AttendanceService {
  /**
   * 计算两个经纬度之间的距离
   * @param {number} lat1 纬度1
   * @param {number} lng1 经度1
   * @param {number} lat2 纬度2
   * @param {number} lng2 经度2
   * @returns {number} 距离，米
   */
  static calculateDistance(lat1, lng1, lat2, lng2) {
    const R = 6371000 // 地球半径，单位：米
    const dLat = (lat2 - lat1) * (Math.PI / 180)
    const dLng = (lng2 - lng1) * (Math.PI / 180)

    const a =
      Math.sin(dLat / 2) * Math.sin(dLat / 2) +
      Math.cos(lat1 * (Math.PI / 180)) *
        Math.cos(lat2 * (Math.PI / 180)) *
        Math.sin(dLng / 2) *
        Math.sin(dLng / 2)

    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a))
    return R * c // 返回单位：米
  }

  /**
   * 判断是否在打卡范围内
   * @param {Object} userLocation 用户位置
   * @param {Object} attendanceGroup 考勤组信息
   * @returns {boolean} 是否在范围内
   */
  static isInAttendanceRange(userLocation, attendanceGroup) {
    const { latitude, longitude } = userLocation
    const { lat, lng, radius } = attendanceGroup

    if (!latitude || !longitude || !lat || !lng) {
      return false
    }

    const distance = this.calculateDistance(latitude, longitude, lat, lng)
    return distance <= radius
  }

  /**
   * 比较当前时间与计划时间
   * @param {string} scheduleTime 计划时间 (HH:mm格式)
   * @returns {number} 比较结果 (-1: 早于, 0: 等于, 1: 晚于)
   */
  static compareTimeWithSchedule(scheduleTime) {
    const now = new Date()
    const [scheduleHour, scheduleMinute] = scheduleTime.split(':').map(Number)

    const currentMinutes = now.getHours() * 60 + now.getMinutes()
    const scheduleMinutes = scheduleHour * 60 + scheduleMinute

    return Math.sign(currentMinutes - scheduleMinutes)
  }

  /**
   * 判断当前时间是否晚于上班时间
   * @param {string} workStartTime 上班时间
   * @returns {boolean} 是否迟到
   */
  static isLateForWork(workStartTime) {
    return this.compareTimeWithSchedule(workStartTime) > 0
  }

  /**
   * 判断当前时间是否早于下班时间
   * @param {string} workEndTime 下班时间
   * @returns {boolean} 是否早退
   */
  static isEarlyForLeaving(workEndTime) {
    return this.compareTimeWithSchedule(workEndTime) < 0
  }

  /**
   * 获取打卡状态文本
   * @param {number} status 状态码
   * @returns {string} 状态文本
   */
  static getStatusText(status) {
    const statusTexts = {
      [PUNCH_STATUS.NOT_CHECKED]: '未打卡',
      [PUNCH_STATUS.CHECKED_IN]: '已打卡',
      [PUNCH_STATUS.CHECKED_OUT]: '已打卡',
      [PUNCH_STATUS.LATE_IN]: '迟到打卡',
      [PUNCH_STATUS.EARLY_OUT]: '早退打卡',
      [PUNCH_STATUS.UPDATE_OUT]: '更新打卡'
    }

    return statusTexts[status] || '未打卡'
  }

  /**
   * 格式化日期时间
   * @param {Date} date 日期对象
   * @returns {string} 格式化后的日期时间
   */
  static formatDateTime(date) {
    return parseTime(date, '{y}-{m}-{d} {h}:{i}:{s}')
  }

  /**
   * 格式化时间显示
   * @param {string} time 时间字符串
   * @returns {string} 格式化后的时间
   */
  static formatTime(time) {
    return time ? parseTime(time, '{h}:{i}:{s}') : ''
  }

  /**
   * 构建打卡请求数据
   * @param {Object} params 参数对象
   * @returns {Object} 打卡数据
   */
  static buildPunchData(params) {
    const { type, userInfo, userLocation, remark } = params

    const statusMap = {
      morning: PUNCH_STATUS.CHECKED_IN,
      evening: PUNCH_STATUS.CHECKED_OUT,
      morning_late: PUNCH_STATUS.LATE_IN,
      evening_early: PUNCH_STATUS.EARLY_OUT
    }

    const isClockIn = type.includes('morning')
    const currentTime = this.formatDateTime(new Date())

    return {
      userId: userInfo.userId,
      userName: userInfo.nickName,
      lng: userLocation.longitude,
      lat: userLocation.latitude,
      checkInStatus: statusMap[type],
      checkInType: isClockIn ? PUNCH_TYPE.CLOCK_IN : PUNCH_TYPE.CLOCK_OUT,
      [isClockIn ? 'clockIn' : 'clockOut']: currentTime,
      description: remark || ''
    }
  }

  /**
   * 处理考勤记录数据
   * @param {Array} records 考勤记录数组
   * @returns {Object} 处理后的考勤状态
   */
  static processAttendanceRecords(records) {
    const attendanceStatus = {
      clockInTime: null,
      clockOutTime: null,
      clockInStatus: PUNCH_STATUS.NOT_CHECKED,
      clockOutStatus: PUNCH_STATUS.NOT_CHECKED
    }

    records.forEach((record) => {
      attendanceStatus.clockInTime = record.clockIn
      attendanceStatus.clockInStatus = parseInt(record.clockInStatus)
      attendanceStatus.clockOutTime = record.clockOut
      attendanceStatus.clockOutStatus = parseInt(record.clockOutStatus || 0)
    })

    return attendanceStatus
  }

  /**
   * 获取考勤组信息
   * @param {string} userId 用户ID
   * @returns {Promise<Object>} 考勤组信息
   */
  static async getAttendanceGroup(userId) {
    const response = await queryAttendanceGroupByUserId({ userId })

    if (response.code !== 200) {
      throw new Error(response.msg || '获取考勤组信息失败')
    }

    if (!response.data) {
      throw new Error('未配置考勤组,请联系管理员')
    }

    return response.data
  }

  /**
   * 获取今日考勤记录
   * @param {string} userId 用户ID
   * @returns {Promise<Array>} 考勤记录数组
   */
  static async getTodayAttendance(userId) {
    const response = await getCurrentDayRecord({ userId })

    if (response.code !== 200) {
      throw new Error(response.msg || '获取考勤状态失败')
    }

    return response.data || []
  }

  /**
   * 提交打卡
   * @param {Object} punchData 打卡数据
   * @returns {Promise<Object>} API响应
   */
  static async submitPunch(punchData) {
    const apiCall =
      punchData.checkInType === PUNCH_TYPE.CLOCK_IN ? checkIn : checkOut
    const response = await apiCall(punchData)

    if (response.code !== 200) {
      throw new Error(response.msg || '打卡失败')
    }

    return response
  }
}
